package widgets

import (
	"fmt"
	"strings"

	"github.com/awesome-gocui/gocui"
)

type ButtonWidget struct {
	Widget
	text string
	h    func(g *gocui.Gui, v *gocui.View) error
}

func NewButtonWidget(g *gocui.Gui, parent *Window, name string, x, y, width int, text string, h func(g *gocui.Gui, v *gocui.View) error) *ButtonWidget {
	return &ButtonWidget{
		Widget: Widget{
			gui:       g,
			parent:    parent,
			name:      name,
			rotatable: true,
			x:         x, y: y,
			width: width, height: 2,
			padding:     0,
			elementType: "button",
		},
		text: text,
		h:    h,
	}
}

func (w *ButtonWidget) Deactivate() {
	w.view.Highlight = false
}

func (w *ButtonWidget) Activate() {
	//w.gui.Cursor = false
	w.view.Highlight = true
	w.gui.SetCurrentView(w.name)
}

func (w *ButtonWidget) SetText(text string) {
	fmt.Fprint(w.view, text)
}

func (w *ButtonWidget) Layout(g *gocui.Gui) error {
	var err error
	if w.parent != nil {
		w.y = w.parent.y + w.parent.currentHeight
		w.x = w.parent.x
		w.width = w.parent.GetWidth() + 1
	}
	w.view, err = g.SetView(w.name, w.x, w.y, w.x+w.width, w.y+w.height, 0)
	if err != nil {
		if err != gocui.ErrUnknownView {
			return err
		}
		canvasSize := w.width - len(w.text)
		paddingLength := canvasSize / 2
		s := w.leftPad(w.text, " ", paddingLength)
		s = w.rightPad(s, " ", paddingLength)
		if len(s) < canvasSize {
			s += ""
		}
		fmt.Fprint(w.view, s)
		w.view.Editable = false
		w.view.BgColor = gocui.ColorBlack
		w.view.FgColor = gocui.ColorWhite
		w.view.SelFgColor = gocui.ColorBlack
		w.view.SelBgColor = gocui.ColorWhite
		w.view.Frame = false
		if w.parent != nil && !w.tabBound {
			w.tabBound = true
			w.gui.SetKeybinding(w.name, gocui.KeyTab, gocui.ModNone, w.parent.Rotator)
			w.gui.SetKeybinding(w.name, gocui.KeyArrowDown, gocui.ModNone, w.parent.Rotator)
			w.gui.SetKeybinding(w.name, gocui.KeyArrowUp, gocui.ModNone, w.parent.RotatorBack)
		}
		w.gui.SetKeybinding(w.name, gocui.KeyEnter, gocui.ModNone, w.h)
		w.gui.SetKeybinding(w.name, gocui.KeySpace, gocui.ModNone, w.h)
	}
	return nil
}

func (w *ButtonWidget) leftPad(s string, padStr string, pLen int) string {
	if pLen > 0 {
		return strings.Repeat(padStr, pLen) + s
	} else {
		return s
	}
}

func (w *ButtonWidget) rightPad(s string, padStr string, pLen int) string {
	if pLen > 0 {
		return s + strings.Repeat(padStr, pLen)
	} else {
		return s
	}
}
