package widgets

import (
	"fmt"
	"github.com/awesome-gocui/gocui"
	"strings"
)

type InputWidget struct {
	Widget
	title string
	text  string
}

func NewInputWidget(g *gocui.Gui, parent *Window, name string, x, y, width int, title string) *InputWidget {
	return &InputWidget{
		Widget: Widget{
			gui:        g,
			parent:     parent,
			name:       name,
			rotatable:  true,
			needCursor: true,
			x:          x, y: y,
			width: width, height: 3,
			elementType: "input",
		},
		title: title,
	}
}

func (w *InputWidget) Deactivate() {
	w.view.Highlight = false
	w.gui.Cursor = false
	//w.gui.Mouse = false
}

func (w *InputWidget) Activate() {
	w.gui.Cursor = true
	//w.gui.Mouse = true
	w.view.Highlight = true
	w.gui.SetCurrentView(w.name)
}

func (w *InputWidget) SetText(text string) {
	w.view.Clear()
	fmt.Fprint(w.view, text)
}

func (w *InputWidget) Layout(g *gocui.Gui) error {
	var err error
	if w.parent != nil {
		w.y = w.parent.y + w.parent.currentHeight
		w.x = w.parent.x + 1
		w.width = w.parent.GetWidth() - 1
	}
	w.view, err = g.SetView(w.name, w.x, w.y, w.x+w.width, w.y+w.height, 0)
	if err != nil {
		if err != gocui.ErrUnknownView {
			return err
		}
		w.view.Title = w.title
		w.view.Editable = true
		w.view.Editor = InputEditor
		w.view.Wrap = false

		w.view.BgColor = gocui.ColorBlack
		w.view.FgColor = gocui.ColorWhite

		w.view.SelBgColor = gocui.ColorBlack
		w.view.SelFgColor = gocui.ColorWhite

		if w.buffer != "" {
			fmt.Fprint(w.view, strings.TrimSpace(w.buffer))
		}

		if w.parent != nil && !w.tabBound {
			w.tabBound = true
			w.gui.SetKeybinding(w.name, gocui.KeyTab, gocui.ModNone, w.parent.Rotator)
			w.gui.SetKeybinding(w.name, gocui.KeyEnter, gocui.ModNone, w.parent.Rotator)
			w.gui.SetKeybinding(w.name, gocui.KeyArrowDown, gocui.ModNone, w.parent.Rotator)
			w.gui.SetKeybinding(w.name, gocui.KeyArrowUp, gocui.ModNone, w.parent.RotatorBack)
		}
	}
	return nil
}
